@@ -2,6 +2,7 @@
 Binds a LibXC Functional struct to a Python object
 """
 
+import sys
 import ctypes
 import numpy as np
 
@@ -14,7 +15,21 @@ from . import structs
 
 # Build out a few common tmps
 _ndptr = np.ctypeslib.ndpointer(dtype=np.double, flags=("C", "A"))
-_ndptr_w = np.ctypeslib.ndpointer(dtype=np.double, flags=("W", "C", "A"))  # Writable
+
+# Workaround to be able to pass NULL pointers to libxc
+# by subclassing the pointer
+_ndptr_w_base = np.ctypeslib.ndpointer(dtype=np.double, flags=("W", "C", "A"))  # Writable
+
+def _from_param(cls, obj):
+    if obj is None:
+        return obj
+    return _ndptr_w_base.from_param(obj)
+
+_ndptr_w = type(
+    'Writable NP Array',
+    (_ndptr_w_base,),
+    {'from_param': classmethod(_from_param)}
+)
 
 _xc_func_p = ctypes.POINTER(structs.xc_func_type)
 _xc_func_info_p = ctypes.POINTER(structs.xc_func_info_type)
@@ -45,18 +60,25 @@ core.xc_func_info_get_flags.argtypes = (_xc_func_info_p, )
 core.xc_func_info_get_references.argtypes = (_xc_func_info_p, ctypes.c_int)
 core.xc_func_info_get_references.restype = ctypes.POINTER(structs.func_reference_type)
 
-# Setters
 core.xc_func_info_get_n_ext_params.argtypes = (_xc_func_info_p, )
 
+core.xc_func_info_get_ext_params_name.argtypes = (_xc_func_info_p, ctypes.c_int)
+core.xc_func_info_get_ext_params_name.restype = ctypes.c_char_p
+
 core.xc_func_info_get_ext_params_description.argtypes = (_xc_func_info_p, ctypes.c_int)
 core.xc_func_info_get_ext_params_description.restype = ctypes.c_char_p
 
 core.xc_func_info_get_ext_params_default_value.argtypes = (_xc_func_info_p, ctypes.c_int)
 core.xc_func_info_get_ext_params_default_value.restype = ctypes.c_double
 
+# Setters
 core.xc_func_set_ext_params.argtypes = (_xc_func_p, _ndptr)
 
-core.xc_func_set_dens_threshold.argtypes = (_xc_func_p, ctypes.c_double)
+# Setters for thresholds
+core.xc_func_set_dens_threshold.argtypes  = (_xc_func_p, ctypes.c_double)
+core.xc_func_set_zeta_threshold.argtypes  = (_xc_func_p, ctypes.c_double)
+core.xc_func_set_sigma_threshold.argtypes = (_xc_func_p, ctypes.c_double)
+core.xc_func_set_tau_threshold.argtypes   = (_xc_func_p, ctypes.c_double)
 
 
 # Bind computers
@@ -64,39 +86,50 @@ def _build_comute_argtype(num_nd, num_nd_write):
     """
     Small function to build the correct argtypes for the LibXC computers
     """
-    ret = [_xc_func_p, ctypes.c_int]
+    ret = [_xc_func_p, ctypes.c_size_t]
     ret += [_ndptr] * num_nd
     ret += [_ndptr_w] * num_nd_write
     return tuple(ret)
 
 
 # LDA computers
-core.xc_lda.argtypes = _build_comute_argtype(1, 4)
+core.xc_lda.argtypes = _build_comute_argtype(1, 5)
 core.xc_lda_exc_vxc.argtypes = _build_comute_argtype(1, 2)
 core.xc_lda_exc.argtypes = _build_comute_argtype(1, 1)
 core.xc_lda_vxc.argtypes = _build_comute_argtype(1, 1)
 core.xc_lda_fxc.argtypes = _build_comute_argtype(1, 1)
 core.xc_lda_kxc.argtypes = _build_comute_argtype(1, 1)
+core.xc_lda_lxc.argtypes = _build_comute_argtype(1, 1)
 
 # GGA computers
-core.xc_gga.argtypes = _build_comute_argtype(2, 10)
+core.xc_gga.argtypes = _build_comute_argtype(2, 15)
 core.xc_gga_exc_vxc.argtypes = _build_comute_argtype(2, 3)
 core.xc_gga_exc.argtypes = _build_comute_argtype(2, 1)
 core.xc_gga_vxc.argtypes = _build_comute_argtype(2, 2)
 core.xc_gga_fxc.argtypes = _build_comute_argtype(2, 3)
 core.xc_gga_kxc.argtypes = _build_comute_argtype(2, 4)
+core.xc_gga_lxc.argtypes = _build_comute_argtype(2, 5)
 
 # MGGA computers
-core.xc_mgga.argtypes = _build_comute_argtype(4, 15)
+core.xc_mgga.argtypes = _build_comute_argtype(4, 70)
 core.xc_mgga_exc_vxc.argtypes = _build_comute_argtype(4, 5)
 core.xc_mgga_exc.argtypes = _build_comute_argtype(4, 1)
 core.xc_mgga_vxc.argtypes = _build_comute_argtype(4, 4)
 core.xc_mgga_fxc.argtypes = _build_comute_argtype(4, 10)
+core.xc_mgga_kxc.argtypes = _build_comute_argtype(4, 20)
+core.xc_mgga_kxc.argtypes = _build_comute_argtype(4, 35)
+
+# hybrid functions
+core.xc_hyb_exx_coef.argtypes = (_xc_func_p, )
+core.xc_hyb_exx_coef.restype  = ctypes.c_double
+
+core.xc_hyb_cam_coef.argtypes = (_xc_func_p, ctypes.POINTER(ctypes.c_double),
+                                 ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double))
+
 
 ### Build LibXCFunctional class
 
-
-def _check_arrays(current_arrays, required, sizes, factor):
+def _check_arrays(current_arrays, fields, sizes, factor, required):
     """
     A specialized function built to construct and check the sizes of arrays given to the LibXCFunctional class.
     """
@@ -104,24 +137,15 @@ def _check_arrays(current_arrays, required, sizes, factor):
     # Nothing supplied so we build it out
     if current_arrays is None:
         current_arrays = {}
-        for label in required:
-            size = sizes["n_" + label]
-            current_arrays[label] = np.zeros((size, factor))
 
-    # Supplied arrays, check sizes
-    else:
-        missing = set(required) - set(current_arrays)
-        if len(missing):
-            raise KeyError("Missing the following output arrays: %s" % ", ".join(missing))
+    for label in fields:
+        if required:
+            size = sizes[label]
+            current_arrays[label] = np.zeros((factor, size))
+        else:
+            current_arrays[label] = None # np.empty((1))
 
-        for label in required:
-            size = sizes["n_" + label] * factor
-            if size != current_arrays[label].size:
-                raise ValueError("Supplied output array '%s' does not have the correct shape number of points by %d" %
-                                 (label, size))
-
-    ret = [current_arrays[x] for x in required]
-    return ret
+    return current_arrays
 
 
 class LibXCFunctional(object):
@@ -167,13 +191,13 @@ class LibXCFunctional(object):
         if isinstance(func_name, str):
             func_id = util.xc_functional_get_number(func_name)
             if func_id == -1:
-                raise KeyError("LibXC Functional name '%s' not found." % func_name)
-        elif isinstance(func_name, int):
+                raise KeyError("LibXCFunctional: name '%s' not found." % func_name)
+        elif isinstance(func_name, (int, np.integer)):
             func_id = func_name
             if util.xc_functional_get_name(func_name) is None:
-                raise KeyError("LibXC Functional ID '%d' not found." % func_name)
+                raise KeyError("LibXCFunctional: ID '%d' not found." % func_name)
         else:
-            raise TypeError("func_name must either be a string or int.")
+            raise TypeError("LibXCFunctional: func_name must either be a string or int. Got {}".format(func_name))
 
         self._xc_func_name = util.xc_functional_get_name(func_id)
 
@@ -185,16 +209,16 @@ class LibXCFunctional(object):
             elif spin == "unpolarized":
                 self._spin = 1
             else:
-                raise KeyError("Spin must either be 'polarized' or 'unpolarized' if represented by a string.")
+                raise KeyError("LibXCFunctional: spin must either be 'polarized' or 'unpolarized' if represented by a string. Got {}".format(spin))
         else:
             self._spin = spin
 
         if self._spin not in [1, 2]:
-            raise KeyError("Spin must either be 1 or 2 if represented by a integer.")
+            raise KeyError("LibXCFunctional: spin must either be 1 or 2 if represented by a integer. Got {}".format(self._spin))
 
         # Build the LibXC functional
         self.xc_func = core.xc_func_alloc()
-        self.xc_func_size_names = [x for x in dir(self.xc_func.contents) if "n_" in x]
+        self.xc_func_size_names = [x for x in dir(self.xc_func.contents.dim) if not "_" in x]
 
         # Set all int attributes to zero (not all set to zero in libxc)
         for attr in self.xc_func_size_names:
@@ -208,7 +232,7 @@ class LibXCFunctional(object):
         # Pull out all sizes after init
         self.xc_func_sizes = {}
         for attr in self.xc_func_size_names:
-            self.xc_func_sizes[attr] = getattr(self.xc_func.contents, attr)
+            self.xc_func_sizes[attr] = getattr(self.xc_func.contents.dim, attr)
 
         # Unpack functional info
         self.xc_func_info = core.xc_func_get_info(self.xc_func)
@@ -239,7 +263,7 @@ class LibXCFunctional(object):
             self._cam_alpha = self.xc_func.contents.cam_alpha
             self._cam_beta = self.xc_func.contents.cam_beta
 
-        elif self._family in [flags.XC_FAMILY_HYB_GGA, flags.XC_FAMILY_HYB_MGGA]:
+        elif self._family in [flags.XC_FAMILY_HYB_LDA, flags.XC_FAMILY_HYB_GGA, flags.XC_FAMILY_HYB_MGGA]:
             self._cam_alpha = self.xc_func.contents.cam_alpha
 
         # VV10
@@ -369,18 +393,22 @@ class LibXCFunctional(object):
         Returns the amount of global exchange to include.
         """
 
-        if self._cam_alpha is False:
-            raise ValueError("Can only be called on Hybrid functionals.")
+        if self._family not in [flags.XC_FAMILY_HYB_LDA, flags.XC_FAMILY_HYB_GGA, flags.XC_FAMILY_HYB_MGGA]:
+            raise ValueError("get_hyb_exx_coef can only be called on hybrid functionals.")
+        if self._have_cam:
+            raise ValueError("get_hyb_exx_coef cannot be called on range-separated functionals.")
 
         return self._cam_alpha
 
     def get_cam_coef(self):
         """
-        Returns the (omega, alpha, beta) quantites
+        Returns the (omega, alpha, beta) quantities
         """
 
-        if self._cam_omega is False:
-            raise ValueError("Can only be called on CAM functionals.")
+        if self._family not in [flags.XC_FAMILY_HYB_LDA, flags.XC_FAMILY_HYB_GGA, flags.XC_FAMILY_HYB_MGGA]:
+            raise ValueError("get_cam_coef can only be called on hybrid functionals.")
+        if not self._have_cam:
+            raise ValueError("get_cam_coef can only be called on range-separated functionals.")
 
         return (self._cam_omega, self._cam_alpha, self._cam_beta)
 
@@ -390,15 +418,28 @@ class LibXCFunctional(object):
         """
 
         if self._nlc_b is False:
-            raise ValueError("Can only be called on -V functionals.")
+            raise ValueError("get_vv10_coeff can only be called on -V functionals.")
 
         return (self._nlc_b, self._nlc_C)
 
     ### Setters
 
+    def get_ext_param_names(self):
+        """
+        Gets the names of all external parameters
+        """
+        num_param = core.xc_func_info_get_n_ext_params(self.xc_func_info)
+
+        ret = []
+        for p in range(num_param):
+            tmp = core.xc_func_info_get_ext_params_name(self.xc_func_info, p)
+            ret.append(tmp.decode("UTF-8"))
+
+        return ret
+
     def get_ext_param_descriptions(self):
         """
-        Gets the description of all external parameters
+        Gets the descriptions of all external parameters
         """
         num_param = core.xc_func_info_get_n_ext_params(self.xc_func_info)
 
@@ -411,7 +452,7 @@ class LibXCFunctional(object):
 
     def get_ext_param_default_values(self):
         """
-        Gets the default value of all external parameters.
+        Gets the default values of all external parameters.
         """
         num_param = core.xc_func_info_get_n_ext_params(self.xc_func_info)
 
@@ -428,18 +469,18 @@ class LibXCFunctional(object):
         """
         num_param = core.xc_func_info_get_n_ext_params(self.xc_func_info)
         if num_param == 0:
-            raise ValueError("The LibXCFunctional '%s' has no extermal parameters to set." % self.get_name())
+            raise ValueError("LibXCFunctional '%s' has no external parameters to set." % self.get_name())
 
         if len(ext_params) != num_param:
             raise ValueError(
-                "The length of the input external parameters (%d) does not match the length of the Functionals external parameters (%d)."
+                "The length of the input external parameters (%d) does not match the length of the functional's external parameters (%d)."
                 % (len(ext_params), num_param))
 
         core.xc_func_set_ext_params(self.xc_func, np.asarray(ext_params, dtype=np.double))
 
     def set_dens_threshold(self, dens_threshold):
         """
-        Sets the density threshold in which densities will not longer be computer.
+        Sets the density threshold below which the functional will not be evaluated.
         """
 
         if dens_threshold < 0:
@@ -447,7 +488,39 @@ class LibXCFunctional(object):
 
         core.xc_func_set_dens_threshold(self.xc_func, ctypes.c_double(dens_threshold))
 
-    def compute(self, inp, output=None, do_exc=True, do_vxc=True, do_fxc=False, do_kxc=False):
+    def set_zeta_threshold(self, zeta_threshold):
+        """
+        Sets the spin polarization threshold below which components will not be evaluated.
+        """
+
+        if zeta_threshold < 0:
+            raise ValueError("The spin polarization threshold cannot be smaller than 0.")
+
+        core.xc_func_set_zeta_threshold(self.xc_func, ctypes.c_double(zeta_threshold))
+
+    def set_sigma_threshold(self, sigma_threshold):
+        """Sets the smallest value allowed for sigma = \sqrt(\gamma). Smaller
+        values than this get overwritten in the evaluation.
+
+        """
+
+        if sigma_threshold < 0:
+            raise ValueError("The sigma threshold cannot be smaller than 0.")
+
+        core.xc_func_set_sigma_threshold(self.xc_func, ctypes.c_double(sigma_threshold))
+
+    def set_tau_threshold(self, tau_threshold):
+        """Sets the smallest value allowed for tau. Smaller values than this
+        get overwritten in the evaluation.
+
+        """
+
+        if tau_threshold < 0:
+            raise ValueError("The tau threshold cannot be smaller than 0.")
+
+        core.xc_func_set_tau_threshold(self.xc_func, ctypes.c_double(tau_threshold))
+
+    def compute(self, inp, output=None, do_exc=True, do_vxc=True, do_fxc=False, do_kxc=False, do_lxc=False):
         """
         Evaluates the functional and its derivatives on a grid.
 
@@ -460,7 +533,7 @@ class LibXCFunctional(object):
                 lapl - the laplacian of the density
                 tau - the kinetic energy density
 
-            Each family of functionals requires different derivates:
+            Each family of functionals requires different derivatives:
                 LDA: rho
                 GGA: rho, sigma
                 MGGA: rho, sigma, lapl (optional), tau
@@ -474,16 +547,36 @@ class LibXCFunctional(object):
                     VCX: vrho
                     FXC: v2rho2
                     KXC: v3rho3
+                    LXC: v4rho4
                 GGA:
                     EXC: zk
                     VCX: vrho, vsigma
                     FXC: v2rho2, v2rhosigma, v2sigma2
                     KXC: v3rho3, v3rho2sigma, v3rhosigma2, v3sigma3
+                    LXC: v4rho4, v4rho3sigma, v4rho2sigma2, v4rhosigma3, v4sigma4
                 MGGA:
                     EXC: zk
                     VCX: vrho, vsigma, vlapl (optional), vtau
+                    FXC: v2rho2, v2rhosigma, v2rholapl, v2rhotau, v2sigma2,
+                         v2sigmalapl, v2sigmatau, v2lapl2, v2lapltau, v2tau2
+                    KXC: v3rho3, v3rho2sigma, v3rho2lapl, v3rho2tau, v3rhosigma2,
+                         v3rhosigmalapl, v3rhosigmatau, v3rholapl2, v3rholapltau,
+                         v3rhotau2, v3sigma3, v3sigma2lapl, v3sigma2tau,
+                         v3sigmalapl2, v3sigmalapltau, v3sigmatau2, v3lapl3,
+                         v3lapl2tau, v3lapltau2, v3tau3
+                    LXC: v4rho4, v4rho3sigma, v4rho3lapl, v4rho3tau, v4rho2sigma2,
+                         v4rho2sigmalapl, v4rho2sigmatau, v4rho2lapl2, v4rho2lapltau,
+                         v4rho2tau2, v4rhosigma3, v4rhosigma2lapl, v4rhosigma2tau,
+                         v4rhosigmalapl2, v4rhosigmalapltau, v4rhosigmatau2,
+                         v4rholapl3, v4rholapl2tau, v4rholapltau2, v4rhotau3,
+                         v4sigma4, v4sigma3lapl, v4sigma3tau, v4sigma2lapl2,
+                         v4sigma2lapltau, v4sigma2tau2, v4sigmalapl3, v4sigmalapl2tau,
+                         v4sigmalapltau2, v4sigmatau3, v4lapl4, v4lapl3tau,
+                         v4lapl2tau2, v4lapltau3, v4tau4
 
-            For unpolarized functional the spin pieces are summed together. However, for polarized functionals the following order will be used for output quantities:
+            For unpolarized functional the spin pieces are summed together.
+            However, for polarized functionals the following order will be used for output quantities:
+            (The last index is the fastest)
 
                 VXC:
                     vrho         = (u, d)
@@ -511,6 +604,8 @@ class LibXCFunctional(object):
             Do evaluate the 2nd derivative of the functional?
         do_kxc : bool (optional, False)
             Do evaluate the 3rd derivative of the functional?
+        do_lxc : bool (optional, False)
+            Do evaluate the 4th derivative of the functional?
 
         Returns
         -------
@@ -543,11 +638,13 @@ class LibXCFunctional(object):
         if not self._have_exc and do_exc:
             raise ValueError("Functional '%s' does not have EXC capabilities." % self.get_name())
         if not self._have_vxc and do_vxc:
-            raise ValueError("Functional '%s' does not have VXC capabilities." % self.get_name())
+            raise ValueError("Functional '%s' does not have VXC capabilities built in." % self.get_name())
         if not self._have_fxc and do_fxc:
-            raise ValueError("Functional '%s' does not have FXC capabilities." % self.get_name())
+            raise ValueError("Functional '%s' does not have FXC capabilities built in." % self.get_name())
         if not self._have_kxc and do_kxc:
-            raise ValueError("Functional '%s' does not have KXC capabilities." % self.get_name())
+            raise ValueError("Functional '%s' does not have KXC capabilities built in." % self.get_name())
+        if not self._have_lxc and do_lxc:
+            raise ValueError("Functional '%s' does not have LXC capabilities built in." % self.get_name())
 
         # Parse input arrays
         if isinstance(inp, np.ndarray):
@@ -563,139 +660,114 @@ class LibXCFunctional(object):
             raise ValueError("Rho input has an invalid shape, must be divisible by %d" % self._spin)
 
         # Find the right compute function
-        args = [self.xc_func, ctypes.c_int(npoints)]
-        if self.get_family() == flags.XC_FAMILY_LDA:
+        args = [self.xc_func, ctypes.c_size_t(npoints)]
+        if self.get_family() in [flags.XC_FAMILY_LDA, flags.XC_FAMILY_HYB_LDA]:
+            input_labels   = ["rho"]
+            input_num_args = 1
+
+            output_labels = [
+                "zk",       # 1, 1
+                "vrho",     # 1, 2
+                "v2rho2",   # 1, 3
+                "v3rho3",   # 1, 4
+                "v4rho4"    # 1, 5
+            ]
 
             # Build input args
-            required_input = ["rho"]
-            args.extend(_check_arrays(inp, required_input, self.xc_func_sizes, npoints))
-            input_num_args = len(args)
+            output = _check_arrays(output, output_labels[0:1],
+                            self.xc_func_sizes, npoints, do_exc)
+            output = _check_arrays(output, output_labels[1:2],
+                            self.xc_func_sizes, npoints, do_vxc)
+            output = _check_arrays(output, output_labels[2:3],
+                            self.xc_func_sizes, npoints, do_fxc)
+            output = _check_arrays(output, output_labels[3:4],
+                            self.xc_func_sizes, npoints, do_kxc)
+            output = _check_arrays(output, output_labels[4:5],
+                            self.xc_func_sizes, npoints, do_lxc)
 
-            # Hybrid computers
-            if do_exc and do_vxc and do_fxc and do_kxc:
-                required_fields = ["zk", "vrho", "v2rho2", "v3rho3"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_lda(*args)
-                do_exc = do_vxc = do_fxc = do_kxc = False
+            args.extend([   inp[x] for x in  input_labels])
+            args.extend([output[x] for x in output_labels])
 
-            if do_exc and do_vxc:
-                required_fields = ["zk", "vrho"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_lda_exc_vxc(*args)
-                do_exc = do_vxc = False
-
-            # Individual computers
-            if do_exc:
-                required_fields = ["zk"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_lda_exc(*args)
-            if do_vxc:
-                required_fields = ["vrho"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_lda_vxc(*args)
-            if do_fxc:
-                required_fields = ["v2rho2"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_lda_fxc(*args)
-            if do_kxc:
-                required_fields = ["v3rho3"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_lda_kxc(*args)
+            core.xc_lda(*args)
 
         elif self.get_family() in [flags.XC_FAMILY_GGA, flags.XC_FAMILY_HYB_GGA]:
+            input_labels   = ["rho", "sigma"]
+            input_num_args = 2
+
+            output_labels = [
+                "zk",                                                               # 1, 1
+                "vrho", "vsigma",                                                   # 2, 3
+                "v2rho2", "v2rhosigma", "v2sigma2",                                 # 3, 6
+                "v3rho3", "v3rho2sigma", "v3rhosigma2", "v3sigma3",                 # 4, 10
+                "v4rho4", "v4rho3sigma", "v4rho2sigma2", "v4rhosigma3", "v4sigma4"  # 5, 15
+            ]
 
             # Build input args
-            required_input = ["rho", "sigma"]
-            args.extend(_check_arrays(inp, required_input, self.xc_func_sizes, npoints))
-            input_num_args = len(args)
+            output = _check_arrays(output, output_labels[0:1],
+                            self.xc_func_sizes, npoints, do_exc)
+            output = _check_arrays(output, output_labels[1:3],
+                            self.xc_func_sizes, npoints, do_vxc)
+            output = _check_arrays(output, output_labels[3:6],
+                            self.xc_func_sizes, npoints, do_fxc)
+            output = _check_arrays(output, output_labels[6:10],
+                            self.xc_func_sizes, npoints, do_kxc)
+            output = _check_arrays(output, output_labels[10:15],
+                            self.xc_func_sizes, npoints, do_lxc)
 
-            # Hybrid computers
-            if do_exc and do_vxc and do_fxc and do_kxc:
-                required_fields = [
-                    "zk", "vrho", "vsigma", "v2rho2", "v2rhosigma", "v2sigma2", "v3rho3", "v3rho2sigma", "v3rhosigma2",
-                    "v3sigma3"
-                ]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_gga(*args)
-                do_exc = do_vxc = do_fxc = do_kxc = False
+            args.extend([   inp[x] for x in  input_labels])
+            args.extend([output[x] for x in output_labels])
 
-            if do_exc and do_vxc:
-                required_fields = ["zk", "vrho", "vsigma"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_gga_exc_vxc(*args)
-                do_exc = do_vxc = False
-
-            # Individual computers
-            if do_exc:
-                required_fields = ["zk"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_gga_exc(*args)
-            if do_vxc:
-                required_fields = ["vrho", "vsigma"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_gga_vxc(*args)
-            if do_fxc:
-                required_fields = ["v2rho2", "v2rhosigma", "v2sigma2"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_gga_fxc(*args)
-            if do_kxc:
-                required_fields = ["v3rho3", "v3rho2sigma", "v3rhosigma2", "v3sigma3"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_gga_kxc(*args)
+            core.xc_gga(*args)
 
         elif self.get_family() in [flags.XC_FAMILY_MGGA, flags.XC_FAMILY_HYB_MGGA]:
             # Build input args
             if self._needs_laplacian:
-                required_input = ["rho", "sigma", "lapl", "tau"]
-                args.extend(_check_arrays(inp, required_input, self.xc_func_sizes, npoints))
+                input_labels = ["rho", "sigma", "lapl", "tau"]
             else:
-                required_input = ["rho", "sigma", "tau"]
-                args.extend(_check_arrays(inp, required_input, self.xc_func_sizes, npoints))
+                input_labels = ["rho", "sigma", "tau"]
+            input_num_args = 4
+
+            output_labels = [
+                "zk",                                                                # 1, 1
+                "vrho", "vsigma", "vlapl", "vtau",                                   # 4, 5
+                "v2rho2", "v2rhosigma", "v2rholapl", "v2rhotau", "v2sigma2",         # 10, 15
+                "v2sigmalapl", "v2sigmatau", "v2lapl2", "v2lapltau",  "v2tau2",
+                "v3rho3", "v3rho2sigma", "v3rho2lapl", "v3rho2tau", "v3rhosigma2",   # 20, 35
+                "v3rhosigmalapl", "v3rhosigmatau", "v3rholapl2", "v3rholapltau",
+                "v3rhotau2", "v3sigma3", "v3sigma2lapl", "v3sigma2tau",
+                "v3sigmalapl2", "v3sigmalapltau", "v3sigmatau2", "v3lapl3",
+                "v3lapl2tau", "v3lapltau2", "v3tau3",
+                "v4rho4", "v4rho3sigma", "v4rho3lapl", "v4rho3tau", "v4rho2sigma2",  # 35, 70
+                "v4rho2sigmalapl", "v4rho2sigmatau", "v4rho2lapl2", "v4rho2lapltau",
+                "v4rho2tau2", "v4rhosigma3", "v4rhosigma2lapl", "v4rhosigma2tau",
+                "v4rhosigmalapl2", "v4rhosigmalapltau", "v4rhosigmatau2",
+                "v4rholapl3", "v4rholapl2tau", "v4rholapltau2", "v4rhotau3",
+                "v4sigma4", "v4sigma3lapl", "v4sigma3tau", "v4sigma2lapl2",
+                "v4sigma2lapltau", "v4sigma2tau2", "v4sigmalapl3", "v4sigmalapl2tau",
+                "v4sigmalapltau2", "v4sigmatau3", "v4lapl4", "v4lapl3tau",
+                "v4lapl2tau2", "v4lapltau3", "v4tau4"
+            ]
+
+            # Build input args
+            output = _check_arrays(output, output_labels[0:1],
+                            self.xc_func_sizes, npoints, do_exc)
+            output = _check_arrays(output, output_labels[1:5],
+                            self.xc_func_sizes, npoints, do_vxc)
+            output = _check_arrays(output, output_labels[5:15],
+                            self.xc_func_sizes, npoints, do_fxc)
+            output = _check_arrays(output, output_labels[15:35],
+                            self.xc_func_sizes, npoints, do_kxc)
+            output = _check_arrays(output, output_labels[35:70],
+                            self.xc_func_sizes, npoints, do_lxc)
+
+            args.extend([   inp[x] for x in  input_labels])
+            if not self._needs_laplacian:
                 args.insert(-1, np.empty((1)))  # Add none ptr to laplacian
-            input_num_args = len(args)
+            args.extend([output[x] for x in output_labels])
 
-            # Hybrid computers
+            core.xc_mgga(*args)
 
-            # Wait until FXC and KXC are available
-            # if do_exc and do_vxc and do_fxc and do_kxc:
-            #     required_fields = [
-            #         "zk", "vrho", "vsigma", "vlapl", "vtau", "v2rho2", "v2sigma2", "v2lapl2", "v2tau2", "v2rhosigma",
-            #         "v2rholapl", "v2rhotau", "v2sigmalapl", "v2sigmatau", "v2lapltau"
-            #     ]
-            #     args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-            #     core.xc_mgga(*args)
-            #     do_exc = do_vxc = do_fxc = do_kxc = False
-
-            if do_exc and do_vxc:
-                required_fields = ["zk", "vrho", "vsigma", "vlapl", "vtau"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_mgga_exc_vxc(*args)
-                do_exc = do_vxc = False
-
-            # Individual computers
-            if do_exc:
-                required_fields = ["zk"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_mgga_exc(*args)
-            if do_vxc:
-                required_fields = ["vrho", "vsigma", "vlapl", "vtau"]
-                args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                core.xc_mgga_vxc(*args)
-            if do_fxc:
-                raise KeyError("FXC quantities (2rd derivitives) are not defined for MGGA's! (%d)")
-                # required_fields = [
-                #     "v2rho2", "v2sigma2", "v2lapl2", "v2tau2", "v2rhosigma", "v2rholapl", "v2rhotau", "v2sigmalapl",
-                #     "v2sigmatau", "v2lapltau"
-                # ]
-                # args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                # core.xc_mgga_fxc(*args)
-            if do_kxc:
-                raise KeyError("KXC quantities (3rd derivitives) are not defined for MGGA's! (%d)")
-                # required_fields = ["v3rho3", "v3rho2sigma", "v3rhosigma2", "v3sigma3"]
-                # args.extend(_check_arrays(output, required_fields, self.xc_func_sizes, npoints))
-                # core.xc_gga_kxc(*args)
         else:
             raise KeyError("Functional kind not recognized! (%d)" % self.get_kind())
 
-        # Return a dictionary
-        return {k: v for k, v in zip(required_fields, args[input_num_args:])}
+        return {k: v for k, v in zip(output_labels, args[2+input_num_args:]) if not v is None}
