@@ -42,21 +42,21 @@ mgga_x_m05_init(xc_func_type *p)
   mgga_x_m05_params *params;
 
   assert(p->params == NULL);
-  p->params = malloc(sizeof(mgga_x_m05_params));
+  p->params = libxc_malloc(sizeof(mgga_x_m05_params));
   params = (mgga_x_m05_params *) (p->params);
 
   switch(p->info->number){
-  case XC_HYB_MGGA_X_M05: 
+  case XC_HYB_MGGA_X_M05:
     memcpy(params, &par_m05, sizeof(mgga_x_m05_params));
-    p->cam_alpha   = 0.28;
+    xc_hyb_init_hybrid(p, 0.28);
     break;
   case XC_HYB_MGGA_X_M05_2X:
     memcpy(params, &par_m05_2x, sizeof(mgga_x_m05_params));
-    p->cam_alpha   = 0.56;
+    xc_hyb_init_hybrid(p, 0.56);
     break;
   case XC_HYB_MGGA_X_M06_2X:
     memcpy(params, &par_m06_2x, sizeof(mgga_x_m05_params));
-    p->cam_alpha   = 0.54;
+    xc_hyb_init_hybrid(p, 0.54);
     break;
   default:
     fprintf(stderr, "Internal error in hyb_mgga_x_m05\n");
@@ -65,48 +65,56 @@ mgga_x_m05_init(xc_func_type *p)
 
 }
 
-#include "maple2c/hyb_mgga_x_m05.c"
-
-#define func maple2c_func
-#include "work_mgga_x.c"
+#include "decl_mgga.h"
+#include "maple2c/mgga_exc/hyb_mgga_x_m05.c"
+#include "work_mgga.c"
 
 
+#ifdef __cplusplus
+extern "C"
+#endif
 const xc_func_info_type xc_func_info_hyb_mgga_x_m05 = {
   XC_HYB_MGGA_X_M05,
   XC_EXCHANGE,
   "Minnesota M05 hybrid exchange functional",
   XC_FAMILY_HYB_MGGA,
   {&xc_ref_Zhao2005_161103, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC,
-  1e-20,
-  0, NULL, NULL,
-  mgga_x_m05_init, NULL, 
-  NULL, NULL, work_mgga_x,
+  XC_FLAGS_3D | MAPLE2C_FLAGS,
+  1e-15,
+  {0, NULL, NULL, NULL, NULL},
+  mgga_x_m05_init, NULL,
+  NULL, NULL, work_mgga,
 };
 
 
+#ifdef __cplusplus
+extern "C"
+#endif
 const xc_func_info_type xc_func_info_hyb_mgga_x_m05_2x = {
   XC_HYB_MGGA_X_M05_2X,
   XC_EXCHANGE,
   "Minnesota M05-2X hybrid exchange functional",
   XC_FAMILY_HYB_MGGA,
   {&xc_ref_Zhao2006_364, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC,
-  1e-20,
-  0, NULL, NULL,
-  mgga_x_m05_init, NULL, 
-  NULL, NULL, work_mgga_x,
+  XC_FLAGS_3D | MAPLE2C_FLAGS,
+  1e-15,
+  {0, NULL, NULL, NULL, NULL},
+  mgga_x_m05_init, NULL,
+  NULL, NULL, work_mgga,
 };
 
+#ifdef __cplusplus
+extern "C"
+#endif
 const xc_func_info_type xc_func_info_hyb_mgga_x_m06_2x = {
   XC_HYB_MGGA_X_M06_2X,
   XC_EXCHANGE,
   "Minnesota M06-2X hybrid exchange functional",
   XC_FAMILY_HYB_MGGA,
   {&xc_ref_Zhao2008_215, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC,
-  1.0e-20,
-  0, NULL, NULL,
+  XC_FLAGS_3D | MAPLE2C_FLAGS,
+  1e-15,
+  {0, NULL, NULL, NULL, NULL},
   mgga_x_m05_init, NULL,
-  NULL, NULL, work_mgga_x,
+  NULL, NULL, work_mgga,
 };
