@@ -10,8 +10,8 @@
 #include "util.h"
 
 /***********************************************************************
-  Exchange and correlation free energy density and potential as 
-  parametrized by 
+  Exchange and correlation free energy density and potential as
+  parametrized by
     Valentin V. Karasiev, Travis Sjostrom, James Dufty, and S. B. Trickey
   Ported to C and libxc by Lazaro Calderin and Miguel Marques
 ************************************************************************/
@@ -20,13 +20,15 @@
 #define XC_LDA_XC_GDSMFB  577    /* Groth et al. parametrization */
 
 typedef struct{
-  double T;
+  double T;            /* In units of k_B */
+  double thetaParam;  /* This takes into account the difference between t and theta_0 */
 
   double b[2][5], c[2][3], d[2][5],  e[2][5];
 } lda_xc_ksdt_params;
 
 static const lda_xc_ksdt_params par_ksdt = {
-  0.0 , /* T */
+  0.0,  /* T */
+  0.0,  /* thetaParam */
   {     /* b5 = Sqrt[3/2]/(lambda)*b3 */
     {0.2839970,  48.9321540, 0.3709190, 61.0953570, 0.871837422702767684673873513724},
     {0.3290010, 111.5983080, 0.5370530,105.0866630, 1.26233194679913807935662124247}
@@ -45,6 +47,7 @@ static const lda_xc_ksdt_params par_ksdt = {
 /* see https://github.com/agbonitz/xc_functional */
 static const lda_xc_ksdt_params par_gdsmfb = {
   0.0 , /* T */
+  0.0,  /* thetaParam */
   {     /* b5 = Sqrt[3/2]/(lambda)*b3 */
     {0.34369020, 7.82159531356, 0.300483986662, 15.8443467125, 0.70628138352268528131},
     {0.84987704, 3.04033012073, 0.0775730131248, 7.57703592489, 0.22972614201992673860}
@@ -58,15 +61,15 @@ static const lda_xc_ksdt_params par_gdsmfb = {
     {0.25388214, 0.815795138599, 0.0646844410481, 15.0984620477, 0.230761357474},
     {0.27454097, 0.400994856555, 2.88773194962, 6.33499237092, 24.823008753}
   }
-}; 
+};
 
-static void 
+static void
 lda_xc_ksdt_init(xc_func_type *p)
-{  
+{
   lda_xc_ksdt_params *params;
 
   assert(p!=NULL && p->params == NULL);
-  p->params = malloc(sizeof(lda_xc_ksdt_params));
+  p->params = libxc_malloc(sizeof(lda_xc_ksdt_params));
   params = (lda_xc_ksdt_params *)(p->params);
 
   switch(p->info->number){
@@ -82,50 +85,55 @@ lda_xc_ksdt_init(xc_func_type *p)
   }
 }
 
-#include "maple2c/lda_xc_ksdt.c"
-
-#define func maple2c_func
+#include "decl_lda.h"
+#include "maple2c/lda_exc/lda_xc_ksdt.c"
 #include "work_lda.c"
 
-static const func_params_type ext_params[] = {
-  {0.0, "Temperature"},
-};
+static const char  *T_names[]  = {"T"};
+static const char  *T_desc[]   = {"Temperature"};
+static const double T_values[] = {0.0};
 
-static void 
-set_ext_params(xc_func_type *p, const double *ext_params)
+static void
+T_set_ext_params(xc_func_type *p, const double *ext_params)
 {
   lda_xc_ksdt_params *params;
-  double ff;
 
   assert(p != NULL && p->params != NULL);
   params = (lda_xc_ksdt_params *) (p->params);
 
-  ff = (ext_params == NULL) ? p->info->ext_params[0].value : ext_params[0];
-  params->T = ff;
+  /* the temperature is in units of k_B */
+  params->T = get_ext_param(p, ext_params, 0);
+  if(params->T < 1e-8) params->T = 1e-8;
 }
 
+#ifdef __cplusplus
+extern "C"
+#endif
 const xc_func_info_type xc_func_info_lda_xc_ksdt = {
   XC_LDA_XC_KSDT,
   XC_EXCHANGE_CORRELATION,
   "Karasiev, Sjostrom, Dufty & Trickey",
   XC_FAMILY_LDA,
   {&xc_ref_Karasiev2014_076403, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC | XC_FLAGS_HAVE_KXC,
-  1e-24,
-  1, ext_params, set_ext_params,
+  XC_FLAGS_3D | MAPLE2C_FLAGS,
+  1e-15,
+  {1, T_names, T_desc, T_values, T_set_ext_params},
   lda_xc_ksdt_init, NULL,
   work_lda, NULL, NULL
 };
 
+#ifdef __cplusplus
+extern "C"
+#endif
 const xc_func_info_type xc_func_info_lda_xc_gdsmfb = {
   XC_LDA_XC_GDSMFB,
   XC_EXCHANGE_CORRELATION,
   "Groth, Dornheim, Sjostrom, Malone, Foulkes, Bonitz",
   XC_FAMILY_LDA,
-  {&xc_ref_Groth2017, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC | XC_FLAGS_HAVE_KXC,
-  1e-24,
-  1, ext_params, set_ext_params,
+  {&xc_ref_Groth2017_135001, NULL, NULL, NULL, NULL},
+  XC_FLAGS_3D | MAPLE2C_FLAGS,
+  1e-15,
+  {1, T_names, T_desc, T_values, T_set_ext_params},
   lda_xc_ksdt_init, NULL,
   work_lda, NULL, NULL
 };
